package pwc.taxtech.atms.vat.service.impl;

import com.google.common.collect.Lists;
import com.mysql.jdbc.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.common.PagingList;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.constant.DataImport;
import pwc.taxtech.atms.constant.enums.EnumTbImportType;
import pwc.taxtech.atms.constant.enums.EnumValidationType;
import pwc.taxtech.atms.constant.enums.VatImportType;
import pwc.taxtech.atms.dto.FieldsMapper;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.PagingDto;
import pwc.taxtech.atms.dto.PagingResultDto;
import pwc.taxtech.atms.dto.vatdto.QueryJeDto;
import pwc.taxtech.atms.dto.vatdto.VoucherDto;
import pwc.taxtech.atms.dto.vatdto.VoucherDtoOnlyForImport;
import pwc.taxtech.atms.vat.dao.JournalEntryImportMapper;
import pwc.taxtech.atms.vat.dao.ValidationInfoMapper;
import pwc.taxtech.atms.vat.dao.VoucherMapper;
import pwc.taxtech.atms.vat.entity.ValidationInfo;
import pwc.taxtech.atms.vat.entity.ValidationInfoExample;
import pwc.taxtech.atms.vat.entity.Voucher;
import pwc.taxtech.atms.vat.entity.VoucherExample;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.UUID;
import java.util.function.ToDoubleFunction;
import java.util.function.ToIntFunction;
import java.util.stream.Collectors;

import static pwc.taxtech.atms.constant.Constant.WholeYear;

@Service
public class JournalEntryImportServiceImpl {
    @Autowired
    JournalEntryImportMapper journalEntryImportMapper;

    @Autowired
    ValidationInfoMapper validationInfoMapper;

    private final static String _tableName = "Voucher";
    private final static String _validationTableName = "ValidationInfo";
    @Autowired
    private VoucherMapper voucherMapper;

    public PagingResultDto<VoucherDto> qeryJournalEntryData(QueryJeDto queryJeDto) {
        PagingResultDto<VoucherDto> qResult = new PagingResultDto<VoucherDto>();
        qResult.setPageInfo(queryJeDto.getPageInfo());
        qResult.setList(new ArrayList<>());
        qResult.setCalculateData(new VoucherDto());

        List<Voucher> pageResultList = journalEntryImportMapper.pageQuery(VatImportType.JournalEntry.getCode(),
                queryJeDto.getPeriodId(), queryJeDto.getPageInfo().getPageIndex() * queryJeDto.getPageInfo().getPageSize(),
                queryJeDto.getPageInfo().getPageSize());

        Integer pageCount = journalEntryImportMapper.pageQueryCount(VatImportType.JournalEntry.getCode(),
                queryJeDto.getPeriodId());

        Integer vIDCount = journalEntryImportMapper.pageVIDCount(VatImportType.JournalEntry.getCode(),
                queryJeDto.getPeriodId());


        //查询结果总数
        qResult.getPageInfo().setTotalCount(pageCount);
        //分录总数
        qResult.getCalculateData().setItemIDCount(pageCount == null ? 0 : pageCount);
        //凭证总数
        qResult.getCalculateData().setvIDCount(vIDCount == null ? 0 : vIDCount);
        //所有凭证的借贷方sum
        BigDecimal debitSum = new BigDecimal(0);
        BigDecimal creditSum = new BigDecimal(0);

        int MonthID = 0, Summary = 0;
        for (Voucher voucher : pageResultList) {
            debitSum.add(voucher.getDebit() != null ? voucher.getDebit() : new BigDecimal(0));
            creditSum.add(voucher.getDebit() != null ? voucher.getCredit() : new BigDecimal(0));

            if (voucher.getMonthID() != null) MonthID = 1;
            if (voucher.getSummary() != null) Summary = 1;
        }

        for (Voucher voucher : pageResultList) {
            if (voucher.getMonthID() == null && MonthID == 1) voucher.setMonthID(0);
            if (voucher.getSummary() == null && Summary == 1) voucher.setSummary("");

            VoucherDto voucherDto = new VoucherDto();
            try {
                FieldsMapper.map(voucher, voucherDto);
            } catch (ClassNotFoundException e) {//TODO replace exception filter (neo)
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
            qResult.getList().add(voucherDto);
        }

        return qResult;
    }

    public List<ValidationInfo> getValidationInfoList(Integer type, Integer period) {
        ValidationInfoExample validationInfoExample = new ValidationInfoExample();
        if (period.intValue() != WholeYear)
            validationInfoExample.createCriteria().andImportTypeIdEqualTo(type).andPeriodIdEqualTo(period);
        else validationInfoExample.createCriteria().andImportTypeIdEqualTo(type);
        return validationInfoMapper.selectByExample(validationInfoExample);
    }

    public void ImportJournalEntryData(List<VoucherDtoOnlyForImport> journalEntryList, Integer importType, String userID) {
        if (journalEntryList.size() > 0) {
            Set<Integer> periods = journalEntryList.stream().collect(Collectors.groupingBy(
                    VoucherDtoOnlyForImport::getPeriod, Collectors.counting())).keySet();

            journalEntryList.forEach(m -> {
                m.VoucherID = UUID.randomUUID().toString();
                m.CreatorID = userID;
                m.CreateTime = new Date();
            });

            if (importType == EnumTbImportType.CoverImport.getCode()) {
                periods.forEach(m -> ClearTableData(m, _tableName));
            }

            journalEntryList.forEach(m -> voucherMapper.insert(m.extractVoucher()));
//            ValidateJournalEntryByPeriods(userID);//todo:[validation] to be continue (neo)
        }
    }

    private void ClearTableData(Integer period, String tableName) {
        switch (tableName) {
            case _tableName:
                VoucherExample example = new VoucherExample();
                example.createCriteria().andPeriodEqualTo(period).andImportTypeEqualTo(VatImportType.JournalEntry.getCode());
                voucherMapper.deleteByExample(period > 0 ? example : null);
                break;
            case _validationTableName:
                ValidationInfoExample example1 = new ValidationInfoExample();
                example1.createCriteria().andPeriodIdEqualTo(period).andImportTypeIdEqualTo(EnumValidationType.JournalEntry.getCode());
                validationInfoMapper.deleteByExample(period > 0 ? example1 : null);
        }
    }

    public OperationResultDto<List<OperationResultDto<List<VoucherDtoOnlyForImport>>>> ImportAuditAdjust(List<VoucherDtoOnlyForImport> voucherList,
                                                                                                         int period, Integer importType, String userID) {
        OperationResultDto<List<OperationResultDto<List<VoucherDtoOnlyForImport>>>> result = new OperationResultDto<>();
        result.setResult(true);

        int num = 1;
        for (VoucherDtoOnlyForImport v : voucherList) {
            v.VoucherID = UUID.randomUUID().toString();
            v.VID = String.format("%sNO%s", new SimpleDateFormat("MMddHHmmss").format(new Date()), num);
            v.Group = v.VID;
            num++;
            v.CreatorID = userID;
            if (v.CreateTime == null) {
                v.date = new Date();
                v.CreateTime = v.date;
            }
            v.ImportType = VatImportType.AuditAdjust.getCode();
        }

        result.setData(new ArrayList<>());

        List<VoucherDtoOnlyForImport> emptyItems = voucherList.stream().filter(dto -> StringUtils.isEmptyOrWhitespaceOnly(dto.AcctCode)
                || dto.Credit == null || dto.Debit == null).collect(Collectors.toList());
        if (emptyItems.size() > 0) {
            result.setResult(false);
            OperationResultDto<List<VoucherDtoOnlyForImport>> rr = new OperationResultDto<>();
            rr.setData(emptyItems);
            rr.setResult(false);
            rr.setResultMsg(DataImport.EmptyField);
            result.getData().add(rr);
        }

        emptyItems = voucherList.stream().filter(dto -> (dto.AcctCode != null && dto.AcctCode.length() > 50)
                || (dto.Summary != null && dto.Summary.length() > 500)).collect(Collectors.toList());
        if (emptyItems.size() > 0) {
            result.setResult(false);
            result.setResult(false);
            OperationResultDto<List<VoucherDtoOnlyForImport>> rr = new OperationResultDto<>();
            rr.setData(emptyItems);
            rr.setResult(false);
            rr.setResultMsg(DataImport.OverLengthField);
            result.getData().add(rr);
        }

        if (result.getResult()) {
            if (importType == EnumTbImportType.CoverImport.getCode()) {
                Set<Integer> perods = voucherList.stream().collect(Collectors.groupingBy(VoucherDtoOnlyForImport::getPeriod,
                        Collectors.counting())).keySet();

                VoucherExample example = new VoucherExample();
                example.createCriteria().andPeriodIn(Lists.newArrayList(period)).andImportTypeEqualTo(VatImportType.AuditAdjust.getCode());
                voucherMapper.deleteByExample(example);
            }

            voucherList.forEach(m -> voucherMapper.insert(m.extractVoucher()));

        }

        return result;
    }

    public List<Voucher> GetAuditAdjust(int period) {
        VoucherExample example = new VoucherExample();
        example.createCriteria().andPeriodEqualTo(period).andImportTypeEqualTo(VatImportType.AuditAdjust.getCode());
        List<Voucher> result = voucherMapper.selectByExample(example);
        result.stream().sorted(Comparator.comparing(Voucher::getAcctCode));
        return result;
    }

    public PagingResultDto<Voucher> QueryAuditAdjust(QueryJeDto queryDto) {
        PagingResultDto<Voucher> qRsult = new PagingResultDto<>();
        qRsult.setPageInfo(queryDto.getPageInfo());
        qRsult.setList(new ArrayList<>());
        qRsult.setCalculateData(new Voucher());
        VoucherExample example = new VoucherExample();
        VoucherExample.Criteria criteria = example.createCriteria();
        criteria.andImportTypeEqualTo(VatImportType.AuditAdjust.getCode());
        if (queryDto.getPeriodId() > Constant.WholeYear) criteria.andPeriodEqualTo(queryDto.getPeriodId());
        List<Voucher> query = voucherMapper.selectByExample(example);
        if(qRsult.getPageInfo()==null)qRsult.setPageInfo(new PagingDto());
        qRsult.getPageInfo().setTotalCount(query.size());
        BigDecimal sumDebit = new BigDecimal(0);
        BigDecimal sumCredit = new BigDecimal(0);
        if (query.size() > 0)
            for (Voucher v : query) {
                sumDebit = sumDebit.add(v.getDebit());
                sumCredit = sumCredit.add(v.getCredit());
            }
        qRsult.getCalculateData().setDebit(sumDebit);
        qRsult.getCalculateData().setCredit(sumCredit);

        if(qRsult.getPageInfo().getTotalCount()>0){
            PagingList pagingList = new PagingList(query, qRsult.getPageInfo());
            qRsult.setList(pagingList.getPagingList());
        }
        return qRsult;
    }
}
