package pwc.taxtech.atms.vat.service.impl;

import org.apache.commons.lang3.StringUtils;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import org.apache.poi.ss.formula.udf.AggregatingUDFFinder;
import org.apache.poi.ss.formula.udf.DefaultUDFFinder;
import org.apache.poi.ss.formula.udf.UDFFinder;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.common.POIUtil;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.enums.CellDataSourceType;
import pwc.taxtech.atms.dao.ProjectMapper;
import pwc.taxtech.atms.dto.vatdto.CellCalcInfoDto;
import pwc.taxtech.atms.dto.vatdto.CellTemplateConfigGroupDto;
import pwc.taxtech.atms.dto.vatdto.PeriodCellTemplateConfigExtendDto;
import pwc.taxtech.atms.entitiy.Project;
import pwc.taxtech.atms.vat.entity.*;
import pwc.taxtech.atms.vat.service.ReportGenerator;
import pwc.taxtech.atms.vat.service.impl.report.functions.*;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import static pwc.taxtech.atms.constant.Constant.EMPTY;

@Component
public class ReportGeneratorImpl extends VatAbstractService implements ReportGenerator {

    private static final Logger logger = LoggerFactory.getLogger(ReportGeneratorImpl.class);
    @Autowired
    private ProjectMapper projectMapper;

    @Autowired
    private FormulaAgent formulaAgent;

    @Override
    public String generateData(String projectId, List<Long> templateIds, Boolean ifDeleteManualDataSource,
                               List<String> exceptCellTemplates, Integer periodParam, String generator) {
        Date createTime = new Date();
        int period = periodParam != null ? periodParam : 0;

        Project project = projectMapper.selectByPrimaryKey(projectId);

        PeriodTemplateExample periodTemplateExample = new PeriodTemplateExample();
        periodTemplateExample.createCriteria().andIdIn(templateIds);
        List<PeriodTemplate> periodTemplateList = periodTemplateMapper.selectByExample(periodTemplateExample);
        List<Long> periodTemplateIDList = periodTemplateList.stream()
                .map(PeriodTemplate::getTemplateId)
                .collect(Collectors.toList());

        Long templateGroupID = periodTemplateList.size() > 0 ? periodTemplateList.get(0).getTemplateGroupId() : 0;

        PeriodCellTemplateExample periodCellTemplateExample = new PeriodCellTemplateExample();
        periodCellTemplateExample.createCriteria().andReportTemplateIdIn(periodTemplateIDList).andPeriodEqualTo(period);
        List<PeriodCellTemplate> periodCellTemplateList = periodCellTemplateMapper.selectByExample(periodCellTemplateExample);

        PeriodCellTemplateConfigExample periodCellTemplateConfigExample = new PeriodCellTemplateConfigExample();
        // only get formula config
        periodCellTemplateConfigExample.createCriteria().andReportTemplateIdIn(periodTemplateIDList).andPeriodEqualTo(period);
        List<PeriodCellTemplateConfig> periodCellTemplateConfigList =
                periodCellTemplateConfigMapper.selectByExample(periodCellTemplateConfigExample);

        //Map<String, String> templateCodeAndPath = new HashMap<>();
        //periodTemplateList.forEach(a -> templateCodeAndPath.put(a.getCode(), getTemplatePath(a.getPath())));
        Optional<Workbook> workbook = createWorkBookWithExcelFileList(periodTemplateList);

        if (workbook.isPresent()) {
            Workbook newWorkbook = workbook.get();
            FormulaContext formulaContext = new FormulaContext();
            formulaContext.setFormulaAgent(formulaAgent);
            formulaContext.setPeriod(period);
            formulaContext.setReportTemplateGroupID(templateGroupID);
            formulaContext.setProjectID(project.getID());
            formulaContext.setYear(project.getYear());
            formulaContext.setOrganizationID(project.getOrganizationID());
            formulaContext.setIfRound(true);

            addFunctionsToWorkbook(newWorkbook, formulaContext);
            int sheetCount = newWorkbook.getNumberOfSheets();
            for (int i = 0; i < sheetCount; i++) {
                Sheet sheet = newWorkbook.getSheetAt(i);
                String code = sheet.getSheetName();
                Optional<PeriodTemplate> periodTemplate = periodTemplateList.stream()
                        .filter(a -> a.getCode().equals(code))
                        .findFirst();
                Long templateID;
                if (periodTemplate.isPresent()) {
                    templateID = periodTemplate.get().getTemplateId();
                } else {
                    templateID = 0L;
                }

                if (templateID > 0) {
                    //get cell template and cell template config with templateID
                    Map<PeriodCellTemplate, PeriodCellTemplateConfig> cellTemplatePeriodCellTemplateConfigMap = new HashMap<>();
                    List<PeriodCellTemplate> tempPeriodCellTemplateList = periodCellTemplateList.stream()
                            .filter(a -> a.getReportTemplateId().equals(templateID))
                            .collect(Collectors.toList());
                    tempPeriodCellTemplateList.forEach(a -> {
                        Optional<PeriodCellTemplateConfig> tempCellTemplateConfig = periodCellTemplateConfigList.stream()
                                .filter(item -> item.getCellTemplateId().equals(a.getCellTemplateId())
                                        && item.getDataSourceType().equals(CellDataSourceType.Formula.getCode()))
                                .findFirst();
                        tempCellTemplateConfig.ifPresent(periodCellTemplateConfig ->
                                cellTemplatePeriodCellTemplateConfigMap.put(a, periodCellTemplateConfig));
                    });

                    cellTemplatePeriodCellTemplateConfigMap.forEach((k, v) -> {
                        Row row = sheet.getRow(k.getRowIndex());
                        if (null == row) {
                            sheet.createRow(k.getRowIndex());
                        }
                        row = sheet.getRow(k.getRowIndex());

                        Cell cell = row.getCell(k.getColumnIndex());
                        if (null == cell) {
                            row.createCell(k.getColumnIndex());
                        }
                        cell = row.getCell(k.getColumnIndex());

                        //todo:后面单独处理kv的公式
                        if (StringUtils.isNotBlank(v.getFormula()) && !v.getFormula().contains("@")) {
                            cell.setCellFormula(v.getFormula());
                            //kv 公式处理
                        } else if (v.getFormula().startsWith("@")) {
                            cell.setCellFormula(v.getKeyValueParsedFormula());
                        }
                    });
                }
            }
            FormulaEvaluator evaluator = newWorkbook.getCreationHelper().createFormulaEvaluator();
            evaluator.evaluateAll();

            //todo:loop all the cell which existing in the cellTemplateConfig, save the data to DB
            for (int i = 0; i < sheetCount; i++) {
                Sheet sheet = newWorkbook.getSheetAt(i);
                String code = sheet.getSheetName();
                Optional<PeriodTemplate> periodTemplate = periodTemplateList.stream()
                        .filter(a -> a.getCode().equals(code))
                        .findFirst();
                Long templateID;
                if (periodTemplate.isPresent()) {
                    templateID = periodTemplate.get().getTemplateId();
                } else {
                    templateID = 0L;
                }

                if (templateID > 0) {
                    //todo: create report into DB
                    Report report = new Report();
                    Long reportID = distributedIDService.nextId();
                    report.setId(reportID);
                    report.setTemplateId(templateID);
                    report.setPeriod(period);
                    report.setProjectId(projectId);
                    report.setCreateBy("Admin");
                    report.setCreateTime(createTime);
                    report.setUpdateBy("Admin");
                    report.setUpdateTime(createTime);
                    reportMapper.insertSelective(report);

                    List<PeriodCellTemplateConfig> periodCellTemplateConfigs = periodCellTemplateConfigList.stream()
                            .filter(a -> a.getReportTemplateId().equals(templateID))
                            .collect(Collectors.toList());

                    //update formulablock table reportid field
                    List<Long> cellTemplateConfigIds = periodCellTemplateConfigs.stream()
                            .map(PeriodCellTemplateConfig::getCellTemplateId)
                            .collect(Collectors.toList());
                    if (cellTemplateConfigIds.size() > 0) {
                        periodFormulaBlockMapper.updateReportId(reportID, cellTemplateConfigIds, period);
                    }

                    for (PeriodCellTemplateConfig periodCellTemplateConfig : periodCellTemplateConfigs) {
                        PeriodFormulaBlockExample periodFormulaBlockExample2 = new PeriodFormulaBlockExample();
                        periodFormulaBlockExample2.createCriteria().andCellTemplateIdEqualTo(periodCellTemplateConfig.getCellTemplateId());
                        List<PeriodFormulaBlock> periodFormulaBlocks = periodFormulaBlockMapper.selectByExample(periodFormulaBlockExample2);

                        //TODO：如果formula 为 ND(100) +ND(22) ,需要使用正则表达式拆分出自定义公式，然后根据自定义公式取formulablock 的数据进行替换
                        String regex = "[A-Z]*\\([A-Za-z0-9\\\"\\,\\.\\u4e00-\\u9fa5\\%]*\\)";
                        Pattern p = Pattern.compile(regex);
                        String sourceFormula = periodCellTemplateConfig.getFormula();
                        String resultFormula = periodCellTemplateConfig.getFormula();
                        Matcher m = p.matcher(sourceFormula);
                        Boolean isFind = false;
                        while (m.find()) {
                            isFind = true;
                            //如果有些公式无法用正则匹配，可以做特殊处理
                            String findStr = m.group();
                            Optional<PeriodFormulaBlock> formulaBlock = periodFormulaBlocks.stream()
                                    .filter(a -> a.getFormulaExpression().equals(findStr))
                                    .findFirst();
                            if (formulaBlock.isPresent()) {
                                resultFormula = resultFormula.replace(findStr, formulaBlock.get().getData());
                            }
                        }

                        //如果有正则匹配就进行更新公式解析
                        if (isFind) {
                            periodCellTemplateConfig.setParsedFormula(resultFormula);
                            periodCellTemplateConfigMapper.updateByPrimaryKey(periodCellTemplateConfig);
                        }

                        String regexNormalCell = "[A-Z]{1,2}((?!0)[0-9]{1,3})";
                        p = Pattern.compile(regexNormalCell);
                        m = p.matcher(sourceFormula);
                        while (m.find()) {
                            //如果找到普通单元格的公式，就去workbook里查找对应的格子取数据，然后放到DataSource，作为普通单元格数据源的数据
                            //找到一个格子就加一条数据
                            String findStr = m.group();//A12,A13,A15,how to get A and 12 or A and 13
                            String regexColumn = "[A-Z]{1,2}";
                            Pattern pp = Pattern.compile(regexColumn);
                            Matcher mm = pp.matcher(findStr);
                            while (mm.find()) {
                                String colStr = mm.group();
                                int colNum = FormulaHelper.excelColStrToNum(colStr, colStr.length());
                                int rowNum = Integer.parseInt(StringUtils.removeStart(findStr, colStr));
                                Row row = sheet.getRow(rowNum);
                                if (row != null) {
                                    Cell cell = row.getCell(colNum);
                                    if (cell != null) {
                                        //开始取值然后存放到DataSource
                                        DataSource dataSource = new DataSource();
                                        dataSource.setId(distributedIDService.nextId());
                                        dataSource.setColumnIndex(colNum);
                                        dataSource.setRowIndex(rowNum);
                                        dataSource.setAmount(new BigDecimal(cell.getNumericCellValue()));
                                        dataSource.setName(findStr);
                                        dataSource.setCreateTime(createTime);
                                        dataSource.setUpdateTime(createTime);
                                        dataSource.setCreateBy("Admin");
                                        dataSource.setUpdateBy("Admin");
                                        dataSource.setPeriod(period);
                                        dataSource.setCellTemplateId(periodCellTemplateConfig.getCellTemplateId());
                                        dataSourceMapper.insertSelective(dataSource);
                                        //这里有个问题就是DataSource的数据有了，但是celldatasource的数据没有，后面无法关联celldata和DataSource
                                        //解决办法就是 在存DataSource的时候就先把celldata的数据加好
                                        //然后把celldatasource的数据也加好
                                        //然后在外面去更新celldata和celldatasource的数据
                                        //还有就是把公式里的数据源放到内存，在所有算完之后再统一加数据源
                                    }
                                }
                            }
                        }

                        Optional<PeriodCellTemplate> tempPeriodCellTemplate = periodCellTemplateList.stream()
                                .filter(a -> a.getCellTemplateId().equals(periodCellTemplateConfig.getCellTemplateId()))
                                .findFirst();
                        if (tempPeriodCellTemplate.isPresent()) {
                            CellData cellData = new CellData();
                            Long cellDataId = distributedIDService.nextId();
                            cellData.setId(cellDataId);
                            cellData.setReportId(reportID);
                            cellData.setCellTemplateId(tempPeriodCellTemplate.get().getCellTemplateId());
                            String data;
                            if (sheet.getRow(tempPeriodCellTemplate.get().getRowIndex()) != null
                                    && sheet.getRow(tempPeriodCellTemplate.get().getRowIndex())
                                    .getCell(tempPeriodCellTemplate.get().getColumnIndex()) != null) {
                                data = Double.toString(sheet.getRow(tempPeriodCellTemplate.get().getRowIndex())
                                        .getCell(tempPeriodCellTemplate.get().getColumnIndex()).getNumericCellValue());
                            } else {
                                data = EMPTY;
                            }
                            cellData.setData(data);
                            PeriodFormulaBlockExample periodFormulaBlockExample = new PeriodFormulaBlockExample();
                            periodFormulaBlockExample.createCriteria().andPeriodEqualTo(period)
                                    .andCellTemplateIdEqualTo(tempPeriodCellTemplate.get().getCellTemplateId());

                            if (isFind) {
                                cellData.setFormulaExp(resultFormula);
                            } else {
                                cellData.setFormulaExp(EMPTY);
                            }
                            cellData.setCreateBy("Admin");
                            cellData.setCreateTime(createTime);
                            cellData.setUpdateBy("Admin");
                            cellData.setUpdateTime(createTime);
                            cellDataMapper.insertSelective(cellData);

                            //after insert celldata, insert the celldatasource for link celldata and datasource
                            DataSourceExample dataSourceExample = new DataSourceExample();
                            dataSourceExample.createCriteria().andPeriodEqualTo(period)
                                    .andCellTemplateIdEqualTo(tempPeriodCellTemplate.get().getCellTemplateId());
                            List<DataSource> dataSourceList = SpringContextUtil.dataSourceMapper.selectByExample(dataSourceExample);
                            for (int ii = 0; ii < dataSourceList.size(); ii++) {
                                DataSource dataSource = dataSourceList.get(ii);
                                CellDataSource cellDataSource = new CellDataSource();
                                cellDataSource.setId(distributedIDService.nextId());
                                cellDataSource.setCellTemplateId(tempPeriodCellTemplate.get().getCellTemplateId());
                                cellDataSource.setCellDataId(cellDataId);
                                cellDataSource.setDataSourceId(dataSource.getId());
                                cellDataSource.setCreateTime(createTime);
                                cellDataSource.setUpdateTime(createTime);
                                SpringContextUtil.cellDataSourceMapper.insertSelective(cellDataSource);
                            }
                        }
                    }
                }
            }

        } else {
            return "GenerateReport failed";
        }
        return "generate report successful";
    }

    private String getTemplatePath(String path) {
        String result;
        String PREFIX_VALUE = "~";
        if (path.startsWith(PREFIX_VALUE)) {
            result = path.substring(1, path.length());
        } else {
            result = path;
        }
        return result;
    }

    /**
     * 根据传入的模板code和模板路径创建workbook
     *
     * @param templates 模板code和模板路径 键值对
     * @return 工作簿workbook
     */
    private Optional<Workbook> createWorkBookWithExcelFileList(List<PeriodTemplate> templates) {
        Workbook workbook = new XSSFWorkbook();
        try {
            String filePath = this.getClass().getResource("").toURI().getPath();
            String tempPath = filePath.substring(0, filePath.indexOf("classes") + "\\classes".length());
            templates.forEach(a -> {
                Workbook tWorkbook = null;
                File file = null;
                if (a.getIsSystemType()) {
                    file = new File(tempPath + a.getPath());
                    try {
                        tWorkbook = WorkbookFactory.create(file);
                    } catch (IOException e) {
                        e.printStackTrace();
                    } catch (InvalidFormatException e) {
                        e.printStackTrace();
                    }
                } else {
                    InputStream is = null;
                    try {
                        is = ftpService.getFtpFileWithStaticUrl(a.getPath());
                        tWorkbook = WorkbookFactory.create(is);
                    } catch (Exception e) {
                        e.printStackTrace();
                    } finally {
                        if (is != null) {
                            try {
                                is.close();
                            } catch (IOException e) {
                                e.printStackTrace();
                            }
                        }
                    }
                }
                POIUtil.cloneSheet(tWorkbook.getSheetAt(0), workbook.createSheet(a.getCode()));
            });
            return Optional.of(workbook);
        } catch (Exception e) {
            logger.error("createWorkBookWithExcelFileList error.", e);
        }
        return Optional.empty();
    }

    /**
     * 注册所有的自定义方法到工作簿
     *
     * @param workbook 工作簿
     */
    private void addFunctionsToWorkbook(Workbook workbook, FormulaContext formulaContext) {
        String[] functionNames = {"SGSR", "FSJZ", "ND", "BB", "XXFP","GZSD"};
        FreeRefFunction[] functionImpls = {new SGSR(formulaContext), new FSJZ(formulaContext), new ND(formulaContext),
                new BB(formulaContext), new XXFP(formulaContext),new GZSD(formulaContext)};
        UDFFinder udfs = new DefaultUDFFinder(functionNames, functionImpls);
        UDFFinder udfToolpack = new AggregatingUDFFinder(udfs);
        workbook.addToolPack(udfToolpack);
    }

    public List<CellCalcInfoDto> getCellCalcInfo(List<Long> templateIDList, Integer periodParam) {
        //already finished by 20180711
        int period = 0;
        if (periodParam != null) {
            period = periodParam;
        }

        List<PeriodCellTemplateConfigExtendDto> periodCellTemplateConfigExtendDtos =
                periodCellTemplateConfigMapper.getPeriodCellTemplateConfigExtendDtos(templateIDList, period);

        List<CellCalcInfoDto> cellCalcInfoDtos = new ArrayList<>();
        periodCellTemplateConfigExtendDtos.stream().collect(Collectors.groupingBy(a ->
                new CellTemplateConfigGroupDto(a.getColumnIndex(), a.getRowIndex()
                        , a.getColumnName(), a.getRowName(), a.getDataType(), a.getIsReadOnly()
                        , a.getCellTemplateID(), a.getReportTemplateID(), a.getComment()))).forEach((x, y) -> {
            CellCalcInfoDto cellCalcInfoDto = new CellCalcInfoDto();
            cellCalcInfoDto.setColumnIndex(x.getColumnIndex());
            cellCalcInfoDto.setColumnName(x.getColumnName());
            cellCalcInfoDto.setRowIndex(x.getRowIndex());
            cellCalcInfoDto.setRowName(x.getRowName());
            cellCalcInfoDto.setFormula(convertListToString(y.stream()
                    .map(PeriodCellTemplateConfig::getFormula).collect(Collectors.toList())));
            cellCalcInfoDto.setParsedFormula(convertListToString(y.stream()
                    .map(PeriodCellTemplateConfig::getParsedFormula).collect(Collectors.toList())));
            //todo: add validation at here later
            cellCalcInfoDto.setCellTemplateID(x.getCellTemplateID());
            cellCalcInfoDto.setReportTemplateID(x.getReportTemplateID());
            cellCalcInfoDto.setDataType(x.getDataType());
            cellCalcInfoDto.setIsReadOnly(x.getIsReadOnly());
            cellCalcInfoDto.setComment(x.getComment());
            cellCalcInfoDto.setConfigList(y.stream()
                    .map(PeriodCellTemplateConfigExtendDto::getPeriodCellTemplateConfig)
                    .collect(Collectors.toList()));
            cellCalcInfoDtos.add(cellCalcInfoDto);
        });
        return cellCalcInfoDtos;
    }

    private String convertListToString(List<String> list) {
        StringBuilder stringBuilder = new StringBuilder();
        list.forEach(s -> {
            if (StringUtils.isNotBlank(s)) {
                stringBuilder.append(s);
                stringBuilder.append("+");
            }
        });
        if (stringBuilder.length() > 0) {
            stringBuilder.deleteCharAt(stringBuilder.length() - 1);
        }
        return stringBuilder.toString();
    }
}
