package pwc.taxtech.atms.vat.service.impl.report.functions;

import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.NumberEval;
import org.apache.poi.ss.formula.eval.StringEval;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import pwc.taxtech.atms.common.util.MyAsserts;
import pwc.taxtech.atms.dto.vatdto.BBParasBo;
import pwc.taxtech.atms.dto.vatdto.CellTemplatePerGroupDto;
import pwc.taxtech.atms.dto.vatdto.CloseableFormulaDataSource;
import pwc.taxtech.atms.dto.vatdto.CurrentPeriodBo;
import pwc.taxtech.atms.dto.vatdto.ReportCellDataSourceDto;
import pwc.taxtech.atms.exception.Exceptions;
import pwc.taxtech.atms.vat.entity.CellData;

import java.math.BigDecimal;

public class BB extends FunctionBase implements FreeRefFunction {
    static final Logger LOGGER = LoggerFactory.getLogger(BB.class);
    final static ValueEval defaultEval = new StringEval("0");
    final static BigDecimal defaultBigDecimal = new BigDecimal("0");

    public BB(FormulaContext context) {
        super(context);
    }

    @Override
    public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
        try {
            return wrapExceptionEval(args, ec);
        } catch (Exception e) {
            LOGGER.warn("un expect exception", e);
            e.printStackTrace();
            return defaultEval;
        }
    }

    public ValueEval wrapExceptionEval(ValueEval[] args, OperationEvaluationContext ec) throws Exception {
        return new NumberEval(bb(new BBParasBo(args, ec)).doubleValue());
    }

    public BigDecimal bb(BBParasBo bo) throws Exception {
        try (CloseableFormulaDataSource closeDataSource = new CloseableFormulaDataSource(formulaContext.getFormulaAgent())) {
            CurrentPeriodBo curPeriod = CurrentPeriodBo.getPeriod(bo.getPeriod().intValue(), formulaContext.getPeriod());
            curPeriod.fixedCurYear(getYear(bo.getYear()));

            ReportCellDataSourceDto nullCellDto = ReportCellDataSourceDto.nullDataSource(bo, curPeriod);
            closeDataSource.addDS(nullCellDto);

            CellTemplatePerGroupDto cellTemplateData = agent.getCellTemplateGroupDto(formulaContext.getReportTemplateGroupID(),
                    formulaContext.getProjectID()).stream().filter(dto -> dto.getRowIndex() == bo.getColumnIndex() - 1
                    && dto.getColumnIndex() == bo.getColumnIndex() - 1).findFirst().get();

            MyAsserts.assertNotNull(cellTemplateData, Exceptions.BB_CELL_TEMP_NULL);
            nullCellDto.fixedWithGroup(cellTemplateData);

            // todo: fix datasource name by templateList(neo)
            if (curPeriod.getCurPeriod() == -99) {
                closeDataSource.clean();
                BigDecimal returnEval = defaultBigDecimal;
                if (formulaContext.getPeriod() <= 1) {
                    return defaultBigDecimal;
                }

                for (int p = 1; p < formulaContext.getPeriod(); p++) {
                    returnEval = returnEval.add(bb(new BBParasBo(bo, p, curPeriod.getCurYear())));
                }

                return returnEval;
            }

            bo.disCount();
            String dbName = agent.getPastProjectDbName(curPeriod.getCurYear(),
                    formulaContext.getOrganizationID());
            CellData cellData = agent.getCellData(dbName, cellTemplateData.getReportTemplateID(),
                    cellTemplateData.getCellTemplateID(), curPeriod.getCurPeriod());
            nullCellDto = ReportCellDataSourceDto.extractFromGroup(bo, curPeriod, cellData, cellTemplateData);
            closeDataSource.clean();
            closeDataSource.addDS(nullCellDto);
            // todo: fix datasource name by templateList(neo)

            MyAsserts.assertNotNull(cellData.getData(), Exceptions.BB_CELL_DATA_NULL);

//                  cellValue=  RoundValue(cellValue, cellDataType)TODO:maybe fixd round by cellDataTyep(KV neo)
            BigDecimal cellValue = new BigDecimal(cellData.getData()).setScale(4,
                    BigDecimal.ROUND_HALF_DOWN);
            nullCellDto.setAmount(cellValue);
            LOGGER.debug("cell static value ");
            return cellValue;
        }
    }

}
